// Compile like this:
//   gcc -std=c99 identify-core.c -o identify-core -Wall -lelf -lebl -ldl '-Wl,-rpath,/usr/$LIB/elfutils/'
#include <elfutils/libebl.h>
#include <error.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>

int
main (int argc, char ** argv)
{
	char const *fn = argv[1];
	errno = 0;
	int fd = open (fn, O_RDONLY);
	if (fd < 0)
		error (1, errno, "open");
	elf_version (EV_CURRENT);
	Elf *elf = elf_begin (fd, ELF_C_READ_MMAP, NULL);
	if (elf == NULL)
		error (1, 0, elf_errmsg (-1));
	Ebl *ebl = ebl_openbackend (elf);
	if (ebl == NULL)
		error (1, 0, "Can't open EBL.");
	GElf_Ehdr ehdr_mem, *ehdr = gelf_getehdr (elf, &ehdr_mem);
	if (ehdr == NULL)
		error (1, 0, elf_errmsg (-1));
	if (ehdr->e_type != ET_CORE)
		error (1, 0, "Not a core file.");

	for (size_t i = 0; i < ehdr->e_phnum; ++i)
		{
			GElf_Phdr phdr_mem, *phdr = gelf_getphdr (elf,
								 i, &phdr_mem);
			if (phdr == NULL || phdr->p_type != PT_NOTE)
				/* Not what we are looking for.  */
				continue;

			GElf_Off start = phdr->p_offset;
			Elf_Data *data = elf_getdata_rawchunk
				(elf, start, phdr->p_filesz, ELF_T_NHDR);

			size_t offset = 0;
			GElf_Nhdr nhdr;
			size_t name_offset;
			size_t desc_offset;
			while (offset < data->d_size
			 && (offset = gelf_getnote (data, offset,
					&nhdr, &name_offset, &desc_offset)) > 0)
	{
		if (nhdr.n_type != NT_PRPSINFO)
			continue;

		GElf_Word regs_offset;
		size_t nregloc;
		const Ebl_Register_Location *reglocs;
		size_t nitems;
		const Ebl_Core_Item *items;

		if (!ebl_core_note (ebl, nhdr.n_type, nhdr.n_descsz,
					&regs_offset, &nregloc, &reglocs,
					&nitems, &items))
			continue;

		const char *desc = data->d_buf + desc_offset;
		for (size_t j = 0; j < nitems; ++j)
			{
				const Ebl_Core_Item *item = items + j;
				if (strcmp (item->name, "fname") == 0
			|| strcmp (item->name, "psargs") == 0)
		printf ("%s\t%s\n", item->name,
			desc + item->offset);
			}

		return 0;
	}
		}
	return 1;
}
